/********************************************************************************
** Form generated from reading UI file 'PrinterBehavior.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PRINTERBEHAVIOR_H
#define UI_PRINTERBEHAVIOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "keditlistwidget.h"
#include "kseparator.h"

QT_BEGIN_NAMESPACE

class Ui_PrinterBehavior
{
public:
    QGridLayout *gridLayout;
    QGroupBox *BanneryGB;
    QGridLayout *gridLayout_5;
    QLabel *label_6;
    QComboBox *startingBannerCB;
    QLabel *label_7;
    QComboBox *endingBannerCB;
    KSeparator *kseparator;
    QGroupBox *PoliciesGB;
    QGridLayout *gridLayout_3;
    QLabel *label_4;
    QLabel *label_3;
    QComboBox *errorPolicyCB;
    QComboBox *operationPolicyCB;
    QGroupBox *AllowGB;
    QGridLayout *gridLayout_6;
    QRadioButton *preventRB;
    QRadioButton *allowRB;
    KEditListWidget *usersELB;

    void setupUi(QWidget *PrinterBehavior)
    {
        if (PrinterBehavior->objectName().isEmpty())
            PrinterBehavior->setObjectName("PrinterBehavior");
        PrinterBehavior->resize(486, 484);
        gridLayout = new QGridLayout(PrinterBehavior);
        gridLayout->setObjectName("gridLayout");
        BanneryGB = new QGroupBox(PrinterBehavior);
        BanneryGB->setObjectName("BanneryGB");
        BanneryGB->setFlat(true);
        gridLayout_5 = new QGridLayout(BanneryGB);
        gridLayout_5->setObjectName("gridLayout_5");
        label_6 = new QLabel(BanneryGB);
        label_6->setObjectName("label_6");
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_5->addWidget(label_6, 0, 0, 1, 1);

        startingBannerCB = new QComboBox(BanneryGB);
        startingBannerCB->setObjectName("startingBannerCB");
        startingBannerCB->setProperty("AttributeName", QVariant(QString::fromUtf8("job-sheets-default")));

        gridLayout_5->addWidget(startingBannerCB, 0, 1, 1, 1);

        label_7 = new QLabel(BanneryGB);
        label_7->setObjectName("label_7");
        label_7->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_5->addWidget(label_7, 1, 0, 1, 1);

        endingBannerCB = new QComboBox(BanneryGB);
        endingBannerCB->setObjectName("endingBannerCB");
        endingBannerCB->setProperty("AttributeName", QVariant(QString::fromUtf8("job-sheets-default")));

        gridLayout_5->addWidget(endingBannerCB, 1, 1, 1, 1);


        gridLayout->addWidget(BanneryGB, 4, 0, 1, 1);

        kseparator = new KSeparator(PrinterBehavior);
        kseparator->setObjectName("kseparator");

        gridLayout->addWidget(kseparator, 5, 0, 1, 1);

        PoliciesGB = new QGroupBox(PrinterBehavior);
        PoliciesGB->setObjectName("PoliciesGB");
        PoliciesGB->setFlat(true);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(PoliciesGB->sizePolicy().hasHeightForWidth());
        PoliciesGB->setSizePolicy(sizePolicy);
        gridLayout_3 = new QGridLayout(PoliciesGB);
        gridLayout_3->setObjectName("gridLayout_3");
        label_4 = new QLabel(PoliciesGB);
        label_4->setObjectName("label_4");
        label_4->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(label_4, 2, 0, 1, 1);

        label_3 = new QLabel(PoliciesGB);
        label_3->setObjectName("label_3");
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_3->addWidget(label_3, 0, 0, 1, 1);

        errorPolicyCB = new QComboBox(PoliciesGB);
        errorPolicyCB->setObjectName("errorPolicyCB");
        errorPolicyCB->setProperty("AttributeName", QVariant(QString::fromUtf8("printer-error-policy")));

        gridLayout_3->addWidget(errorPolicyCB, 0, 1, 1, 1);

        operationPolicyCB = new QComboBox(PoliciesGB);
        operationPolicyCB->setObjectName("operationPolicyCB");
        operationPolicyCB->setProperty("AttributeName", QVariant(QString::fromUtf8("printer-op-policy")));

        gridLayout_3->addWidget(operationPolicyCB, 2, 1, 1, 1);


        gridLayout->addWidget(PoliciesGB, 1, 0, 1, 1);

        AllowGB = new QGroupBox(PrinterBehavior);
        AllowGB->setObjectName("AllowGB");
        AllowGB->setFlat(true);
        gridLayout_6 = new QGridLayout(AllowGB);
        gridLayout_6->setObjectName("gridLayout_6");
        preventRB = new QRadioButton(AllowGB);
        preventRB->setObjectName("preventRB");

        gridLayout_6->addWidget(preventRB, 1, 0, 1, 1);

        allowRB = new QRadioButton(AllowGB);
        allowRB->setObjectName("allowRB");

        gridLayout_6->addWidget(allowRB, 0, 0, 1, 1);

        usersELB = new KEditListWidget(AllowGB);
        usersELB->setObjectName("usersELB");
        usersELB->setButtons(KEditListWidget::Add|KEditListWidget::Remove);

        gridLayout_6->addWidget(usersELB, 2, 0, 1, 1);


        gridLayout->addWidget(AllowGB, 7, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label_6->setBuddy(startingBannerCB);
        label_7->setBuddy(endingBannerCB);
        label_4->setBuddy(operationPolicyCB);
        label_3->setBuddy(errorPolicyCB);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(errorPolicyCB, operationPolicyCB);
        QWidget::setTabOrder(operationPolicyCB, startingBannerCB);
        QWidget::setTabOrder(startingBannerCB, endingBannerCB);
        QWidget::setTabOrder(endingBannerCB, allowRB);
        QWidget::setTabOrder(allowRB, preventRB);

        retranslateUi(PrinterBehavior);

        QMetaObject::connectSlotsByName(PrinterBehavior);
    } // setupUi

    void retranslateUi(QWidget *PrinterBehavior)
    {
        BanneryGB->setTitle(tr2i18n("Banners", nullptr));
        label_6->setText(tr2i18n("Starting Banner:", nullptr));
        label_7->setText(tr2i18n("Ending Banner:", nullptr));
        PoliciesGB->setTitle(tr2i18n("Policies", nullptr));
        label_4->setText(tr2i18n("Operation Policy:", nullptr));
        label_3->setText(tr2i18n("Error Policy:", nullptr));
        AllowGB->setTitle(tr2i18n("Allowed Users", nullptr));
        preventRB->setText(tr2i18n("Pre&vent these users from printing ", nullptr));
        allowRB->setText(tr2i18n("A&llow these users to print", nullptr));
        (void)PrinterBehavior;
    } // retranslateUi

};

namespace Ui {
    class PrinterBehavior: public Ui_PrinterBehavior {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PRINTERBEHAVIOR_H

