/****************************************************************************
** Meta object code from reading C++ file 'JobModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../print-manager-6.6.5/src/libkcups/JobModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'JobModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8JobModelE_t {};
} // unnamed namespace

template <> constexpr inline auto JobModel::qt_create_metaobjectdata<qt_meta_tag_ZN8JobModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "JobModel",
        "QML.Element",
        "auto",
        "error",
        "",
        "lastError",
        "errorTitle",
        "errorMsg",
        "getJobs",
        "getJobFinished",
        "KCupsRequest*",
        "request",
        "jobCompleted",
        "text",
        "printerUri",
        "printerName",
        "printerState",
        "printerStateReasons",
        "printerIsAcceptingJobs",
        "jobId",
        "jobState",
        "jobStateReasons",
        "jobName",
        "jobImpressionsCompleted",
        "insertUpdateJob",
        "init",
        "destName",
        "hold",
        "release",
        "cancel",
        "move",
        "toPrinterName",
        "setWhichJobs",
        "WhichJobs",
        "whichjobs",
        "Role",
        "RoleJobId",
        "RoleJobState",
        "RoleJobName",
        "RoleJobPages",
        "RoleJobSize",
        "RoleJobOwner",
        "RoleJobCreatedAt",
        "RoleJobIconName",
        "RoleJobCancelEnabled",
        "RoleJobHoldEnabled",
        "RoleJobReleaseEnabled",
        "RoleJobRestartEnabled",
        "RoleJobPrinter",
        "RoleJobOriginatingHostName",
        "RoleJobAuthenticationRequired",
        "JobAction",
        "Cancel",
        "Hold",
        "Release",
        "Move",
        "Reprint",
        "WhichAll",
        "WhichActive",
        "WhichCompleted"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'error'
        QtMocHelpers::SignalData<void(int, const QString &, const QString &)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 }, { QMetaType::QString, 6 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'getJobs'
        QtMocHelpers::SlotData<void()>(8, 4, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'getJobFinished'
        QtMocHelpers::SlotData<void(KCupsRequest *)>(9, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'jobCompleted'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool, uint, uint, const QString &, const QString &, uint)>(12, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 }, { QMetaType::UInt, 16 },
            { QMetaType::QString, 17 }, { QMetaType::Bool, 18 }, { QMetaType::UInt, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::QString, 22 }, { QMetaType::UInt, 23 },
        }}),
        // Slot 'insertUpdateJob'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool, uint, uint, const QString &, const QString &, uint)>(24, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 }, { QMetaType::UInt, 16 },
            { QMetaType::QString, 17 }, { QMetaType::Bool, 18 }, { QMetaType::UInt, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::QString, 22 }, { QMetaType::UInt, 23 },
        }}),
        // Method 'init'
        QtMocHelpers::MethodData<void(const QString &)>(25, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Method 'init'
        QtMocHelpers::MethodData<void()>(25, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Method 'hold'
        QtMocHelpers::MethodData<void(const QString &, int)>(27, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Int, 19 },
        }}),
        // Method 'release'
        QtMocHelpers::MethodData<void(const QString &, int)>(28, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Int, 19 },
        }}),
        // Method 'cancel'
        QtMocHelpers::MethodData<void(const QString &, int)>(29, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Int, 19 },
        }}),
        // Method 'move'
        QtMocHelpers::MethodData<void(const QString &, int, const QString &)>(30, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Int, 19 }, { QMetaType::QString, 31 },
        }}),
        // Method 'setWhichJobs'
        QtMocHelpers::MethodData<void(enum WhichJobs)>(32, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 33, 34 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Role'
        QtMocHelpers::EnumData<enum Role>(35, 35, QMC::EnumFlags{}).add({
            {   36, Role::RoleJobId },
            {   37, Role::RoleJobState },
            {   38, Role::RoleJobName },
            {   39, Role::RoleJobPages },
            {   40, Role::RoleJobSize },
            {   41, Role::RoleJobOwner },
            {   42, Role::RoleJobCreatedAt },
            {   43, Role::RoleJobIconName },
            {   44, Role::RoleJobCancelEnabled },
            {   45, Role::RoleJobHoldEnabled },
            {   46, Role::RoleJobReleaseEnabled },
            {   47, Role::RoleJobRestartEnabled },
            {   48, Role::RoleJobPrinter },
            {   49, Role::RoleJobOriginatingHostName },
            {   50, Role::RoleJobAuthenticationRequired },
        }),
        // enum 'JobAction'
        QtMocHelpers::EnumData<enum JobAction>(51, 51, QMC::EnumFlags{}).add({
            {   52, JobAction::Cancel },
            {   53, JobAction::Hold },
            {   54, JobAction::Release },
            {   55, JobAction::Move },
            {   56, JobAction::Reprint },
        }),
        // enum 'WhichJobs'
        QtMocHelpers::EnumData<enum WhichJobs>(33, 33, QMC::EnumFlags{}).add({
            {   57, WhichJobs::WhichAll },
            {   58, WhichJobs::WhichActive },
            {   59, WhichJobs::WhichCompleted },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<JobModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject JobModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QStandardItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8JobModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8JobModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8JobModelE_t>.metaTypes,
    nullptr
} };

void JobModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<JobModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->error((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 1: _t->getJobs(); break;
        case 2: _t->getJobFinished((*reinterpret_cast<std::add_pointer_t<KCupsRequest*>>(_a[1]))); break;
        case 3: _t->jobCompleted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[11]))); break;
        case 4: _t->insertUpdateJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[7])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[8])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[9])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[10])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[11]))); break;
        case 5: _t->init((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->init(); break;
        case 7: _t->hold((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 8: _t->release((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 9: _t->cancel((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->move((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 11: _t->setWhichJobs((*reinterpret_cast<std::add_pointer_t<enum WhichJobs>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (JobModel::*)(int , const QString & , const QString & )>(_a, &JobModel::error, 0))
            return;
    }
}

const QMetaObject *JobModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *JobModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8JobModelE_t>.strings))
        return static_cast<void*>(this);
    return QStandardItemModel::qt_metacast(_clname);
}

int JobModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStandardItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void JobModel::error(int _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
