/*
 * This file was generated by qdbusxml2cpp version 0.8
 * Source file was com.redhat.NewPrinterNotification.xml
 *
 * qdbusxml2cpp is Copyright (C) The Qt Company Ltd. and other contributors.
 *
 * This is an auto-generated file.
 * Do not edit! All changes made to it will be lost.
 */

#include "newprinternotificationadaptor.h"
#include <QtCore/QMetaObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

/*
 * Implementation of adaptor class NewPrinterNotificationAdaptor
 */

NewPrinterNotificationAdaptor::NewPrinterNotificationAdaptor(NewPrinterNotification *parent)
    : QDBusAbstractAdaptor(parent)
{
    // constructor
    setAutoRelaySignals(true);
}

NewPrinterNotificationAdaptor::~NewPrinterNotificationAdaptor()
{
    // destructor
}

void NewPrinterNotificationAdaptor::GetReady()
{
    // handle method call com.redhat.NewPrinterNotification.GetReady
    parent()->GetReady();
}

void NewPrinterNotificationAdaptor::NewPrinter(int status, const QString &name, const QString &mfg, const QString &mdl, const QString &des, const QString &cmd)
{
    // handle method call com.redhat.NewPrinterNotification.NewPrinter
    parent()->NewPrinter(status, name, mfg, mdl, des, cmd);
}


#include "moc_newprinternotificationadaptor.cpp"
