/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<DevicesModel.h>)
#  include <DevicesModel.h>
#endif
#if __has_include(<JobModel.h>)
#  include <JobModel.h>
#endif
#if __has_include(<JobSortFilterModel.h>)
#  include <JobSortFilterModel.h>
#endif
#if __has_include(<PPDModel.h>)
#  include <PPDModel.h>
#endif
#if __has_include(<PrinterModel.h>)
#  include <PrinterModel.h>
#endif
#if __has_include(<ProcessRunner.h>)
#  include <ProcessRunner.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_org_kde_plasma_printmanager()
{
    qmlRegisterModule("org.kde.plasma.printmanager", 254, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<DevicesModel>("org.kde.plasma.printmanager", 254);
    qmlRegisterEnum<DevicesModel::Role>("DevicesModel::Role");
    qmlRegisterEnum<DevicesModel::Kind>("DevicesModel::Kind");
    qmlRegisterTypesAndRevisions<JobModel>("org.kde.plasma.printmanager", 254);
    qmlRegisterEnum<JobModel::Role>("JobModel::Role");
    qmlRegisterEnum<JobModel::JobAction>("JobModel::JobAction");
    qmlRegisterEnum<JobModel::WhichJobs>("JobModel::WhichJobs");
    qmlRegisterTypesAndRevisions<JobSortFilterModel>("org.kde.plasma.printmanager", 254);
    qmlRegisterTypesAndRevisions<PPDModel>("org.kde.plasma.printmanager", 254);
    qmlRegisterEnum<PPDModel::Role>("PPDModel::Role");
    qmlRegisterTypesAndRevisions<PrinterModel>("org.kde.plasma.printmanager", 254);
    qmlRegisterEnum<PrinterModel::Role>("PrinterModel::Role");
    qmlRegisterEnum<PrinterModel::JobAction>("PrinterModel::JobAction");
    qmlRegisterTypesAndRevisions<ProcessRunner>("org.kde.plasma.printmanager", 254);
    QMetaType::fromType<QAbstractItemModel *>().id();
    qmlRegisterEnum<QAbstractItemModel::LayoutChangeHint>("QAbstractItemModel::LayoutChangeHint");
    qmlRegisterEnum<QAbstractItemModel::CheckIndexOption>("QAbstractItemModel::CheckIndexOption");
    QMetaType::fromType<QAbstractProxyModel *>().id();
    QMetaType::fromType<QSortFilterProxyModel *>().id();
    QMetaType::fromType<QStandardItemModel *>().id();
    QT_WARNING_POP
    qmlRegisterModule("org.kde.plasma.printmanager", 254, 254);
}

static const QQmlModuleRegistration orgkdeplasmaprintmanagerRegistration("org.kde.plasma.printmanager", qml_register_types_org_kde_plasma_printmanager);
