/********************************************************************************
** Form generated from reading UI file 'PrintQueueUi.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PRINTQUEUEUI_H
#define UI_PRINTQUEUEUI_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kseparator.h"

QT_BEGIN_NAMESPACE

class Ui_PrintQueueUi
{
public:
    QGridLayout *gridLayout;
    QGridLayout *gridLayout_2;
    QLabel *iconL;
    QSpacerItem *verticalSpacer_2;
    QLabel *statusL;
    QLabel *printerStatusMsgL;
    QPushButton *pausePrinterPB;
    QPushButton *configurePrinterPB;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout;
    KSeparator *kseparator;
    QTreeView *jobsView;
    QHBoxLayout *horizontalLayout_2;
    KSeparator *kseparator1;
    QHBoxLayout *horizontalLayout;
    QPushButton *cancelJobPB;
    QFrame *line;
    QPushButton *holdJobPB;
    QPushButton *resumeJobPB;
    QPushButton *reprintPB;
    QSpacerItem *horizontalSpacer_2;
    QComboBox *whichJobsCB;

    void setupUi(QWidget *PrintQueueUi)
    {
        if (PrintQueueUi->objectName().isEmpty())
            PrintQueueUi->setObjectName("PrintQueueUi");
        PrintQueueUi->resize(550, 402);
        PrintQueueUi->setMinimumSize(QSize(550, 0));
        gridLayout = new QGridLayout(PrintQueueUi);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        gridLayout_2->setContentsMargins(10, -1, -1, -1);
        iconL = new QLabel(PrintQueueUi);
        iconL->setObjectName("iconL");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(iconL->sizePolicy().hasHeightForWidth());
        iconL->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(iconL, 0, 0, 5, 1);

        verticalSpacer_2 = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Ignored);

        gridLayout_2->addItem(verticalSpacer_2, 0, 1, 1, 1);

        statusL = new QLabel(PrintQueueUi);
        statusL->setObjectName("statusL");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(statusL->sizePolicy().hasHeightForWidth());
        statusL->setSizePolicy(sizePolicy1);
        QFont font;
        font.setPointSize(12);
        statusL->setFont(font);
        statusL->setText(QString::fromUtf8("Printer Status"));
        statusL->setWordWrap(true);

        gridLayout_2->addWidget(statusL, 1, 1, 1, 4);

        printerStatusMsgL = new QLabel(PrintQueueUi);
        printerStatusMsgL->setObjectName("printerStatusMsgL");
        sizePolicy1.setHeightForWidth(printerStatusMsgL->sizePolicy().hasHeightForWidth());
        printerStatusMsgL->setSizePolicy(sizePolicy1);
        printerStatusMsgL->setText(QString::fromUtf8("printer status message"));
        printerStatusMsgL->setWordWrap(true);
        printerStatusMsgL->setIndent(10);

        gridLayout_2->addWidget(printerStatusMsgL, 2, 1, 1, 4);

        pausePrinterPB = new QPushButton(PrintQueueUi);
        pausePrinterPB->setObjectName("pausePrinterPB");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(pausePrinterPB->sizePolicy().hasHeightForWidth());
        pausePrinterPB->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(pausePrinterPB, 4, 1, 1, 1);

        configurePrinterPB = new QPushButton(PrintQueueUi);
        configurePrinterPB->setObjectName("configurePrinterPB");
        sizePolicy2.setHeightForWidth(configurePrinterPB->sizePolicy().hasHeightForWidth());
        configurePrinterPB->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(configurePrinterPB, 4, 2, 1, 1);

        verticalSpacer = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Ignored);

        gridLayout_2->addItem(verticalSpacer, 3, 1, 1, 1);


        gridLayout->addLayout(gridLayout_2, 0, 0, 1, 2);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(0);
        verticalLayout->setObjectName("verticalLayout");
        kseparator = new KSeparator(PrintQueueUi);
        kseparator->setObjectName("kseparator");

        verticalLayout->addWidget(kseparator);

        jobsView = new QTreeView(PrintQueueUi);
        jobsView->setObjectName("jobsView");
        jobsView->setMinimumSize(QSize(0, 130));
        jobsView->setBaseSize(QSize(640, 0));
        jobsView->setContextMenuPolicy(Qt::CustomContextMenu);
        jobsView->setAcceptDrops(true);
        jobsView->setFrameShape(QFrame::NoFrame);
        jobsView->setProperty("showDropIndicator", QVariant(false));
        jobsView->setAlternatingRowColors(true);
        jobsView->setSelectionMode(QAbstractItemView::ExtendedSelection);
        jobsView->setTextElideMode(Qt::ElideMiddle);
        jobsView->setRootIsDecorated(false);
        jobsView->setSortingEnabled(true);
        jobsView->setAnimated(true);
        jobsView->setExpandsOnDoubleClick(false);

        verticalLayout->addWidget(jobsView);


        gridLayout->addLayout(verticalLayout, 3, 0, 1, 2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        kseparator1 = new KSeparator(PrintQueueUi);
        kseparator1->setObjectName("kseparator1");

        horizontalLayout_2->addWidget(kseparator1);


        gridLayout->addLayout(horizontalLayout_2, 1, 0, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(4, -1, 4, -1);
        cancelJobPB = new QPushButton(PrintQueueUi);
        cancelJobPB->setObjectName("cancelJobPB");
        cancelJobPB->setEnabled(false);
        cancelJobPB->setFlat(true);

        horizontalLayout->addWidget(cancelJobPB);

        line = new QFrame(PrintQueueUi);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::VLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        horizontalLayout->addWidget(line);

        holdJobPB = new QPushButton(PrintQueueUi);
        holdJobPB->setObjectName("holdJobPB");
        holdJobPB->setVisible(false);
        holdJobPB->setFlat(true);

        horizontalLayout->addWidget(holdJobPB);

        resumeJobPB = new QPushButton(PrintQueueUi);
        resumeJobPB->setObjectName("resumeJobPB");
        resumeJobPB->setVisible(false);
        resumeJobPB->setFlat(true);

        horizontalLayout->addWidget(resumeJobPB);

        reprintPB = new QPushButton(PrintQueueUi);
        reprintPB->setObjectName("reprintPB");
        reprintPB->setEnabled(false);
        reprintPB->setFlat(true);

        horizontalLayout->addWidget(reprintPB);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);

        whichJobsCB = new QComboBox(PrintQueueUi);
        whichJobsCB->addItem(QString());
        whichJobsCB->addItem(QString());
        whichJobsCB->addItem(QString());
        whichJobsCB->setObjectName("whichJobsCB");
        whichJobsCB->setInsertPolicy(QComboBox::NoInsert);
        whichJobsCB->setSizeAdjustPolicy(QComboBox::AdjustToContents);
        whichJobsCB->setFrame(false);

        horizontalLayout->addWidget(whichJobsCB);


        gridLayout->addLayout(horizontalLayout, 2, 0, 1, 2);


        retranslateUi(PrintQueueUi);

        QMetaObject::connectSlotsByName(PrintQueueUi);
    } // setupUi

    void retranslateUi(QWidget *PrintQueueUi)
    {
        PrintQueueUi->setWindowTitle(tr2i18n("my printer (x jobs)", nullptr));
        iconL->setText(tr2i18n("Printer Icon", nullptr));
        pausePrinterPB->setText(tr2i18n("Pause Printer", nullptr));
        configurePrinterPB->setText(tr2i18n("Configure", nullptr));
        cancelJobPB->setText(tr2i18n("Cancel", nullptr));
        holdJobPB->setText(tr2i18n("Hold", nullptr));
        resumeJobPB->setText(tr2i18n("Resume", nullptr));
        reprintPB->setText(tr2i18n("Reprint", nullptr));
        whichJobsCB->setItemText(0, tr2i18n("Active Jobs", nullptr));
        whichJobsCB->setItemText(1, tr2i18n("Completed Jobs", nullptr));
        whichJobsCB->setItemText(2, tr2i18n("All Jobs", nullptr));

    } // retranslateUi

};

namespace Ui {
    class PrintQueueUi: public Ui_PrintQueueUi {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PRINTQUEUEUI_H

