/*
    SPDX-FileCopyrightText: 2014 Jan Grulich <jgrulich@redhat.com>

    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
*/

#ifndef LIBKCUPS_DEBUG_H
#define LIBKCUPS_DEBUG_H

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(PM_CONFIGURE_PRINTER)

#endif // LIBKCUPS_DEBUG_H
