/****************************************************************************
** Meta object code from reading C++ file 'PrinterModel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../print-manager-6.6.5/src/libkcups/PrinterModel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PrinterModel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12PrinterModelE_t {};
} // unnamed namespace

template <> constexpr inline auto PrinterModel::qt_create_metaobjectdata<qt_meta_tag_ZN12PrinterModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PrinterModel",
        "QML.Element",
        "auto",
        "serverUnavailableChanged",
        "",
        "unavailable",
        "error",
        "lastError",
        "errorTitle",
        "errorMsg",
        "showLocationsChanged",
        "hasOnlyPrintersChanged",
        "update",
        "getDestsFinished",
        "KCupsRequest*",
        "request",
        "insertUpdatePrinter",
        "text",
        "printerUri",
        "printerName",
        "printerState",
        "printerStateReasons",
        "printerIsAcceptingJobs",
        "insertUpdatePrinterFinished",
        "printerRemoved",
        "printerStateChanged",
        "printerStopped",
        "printerRestarted",
        "printerShutdown",
        "printerModified",
        "serverChanged",
        "pausePrinter",
        "resumePrinter",
        "serverUnavailable",
        "showLocations",
        "hasOnlyPrinters",
        "Role",
        "DestStatus",
        "DestState",
        "DestName",
        "DestIsDefault",
        "DestIsShared",
        "DestIsAcceptingJobs",
        "DestIsPaused",
        "DestIsClass",
        "DestLocation",
        "DestDescription",
        "DestKind",
        "DestType",
        "DestCommands",
        "DestMarkerChangeTime",
        "DestMarkers",
        "DestIconName",
        "DestRemote",
        "DestUri",
        "DestUriSupported",
        "DestMemberNames",
        "JobAction",
        "Cancel",
        "Hold",
        "Release",
        "Move"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'serverUnavailableChanged'
        QtMocHelpers::SignalData<void(bool)>(3, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(int, const QString &, const QString &)>(6, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::QString, 8 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'showLocationsChanged'
        QtMocHelpers::SignalData<void()>(10, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hasOnlyPrintersChanged'
        QtMocHelpers::SignalData<void()>(11, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'update'
        QtMocHelpers::SlotData<void()>(12, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getDestsFinished'
        QtMocHelpers::SlotData<void(KCupsRequest *)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'insertUpdatePrinter'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(16, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'insertUpdatePrinterFinished'
        QtMocHelpers::SlotData<void(KCupsRequest *)>(23, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Slot 'printerRemoved'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(24, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'printerStateChanged'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(25, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'printerStopped'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(26, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'printerRestarted'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(27, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'printerShutdown'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(28, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'printerModified'
        QtMocHelpers::SlotData<void(const QString &, const QString &, const QString &, uint, const QString &, bool)>(29, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::UInt, 20 },
            { QMetaType::QString, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'serverChanged'
        QtMocHelpers::SlotData<void(const QString &)>(30, 4, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Method 'pausePrinter'
        QtMocHelpers::MethodData<void(const QString &)>(31, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Method 'resumePrinter'
        QtMocHelpers::MethodData<void(const QString &)>(32, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'serverUnavailable'
        QtMocHelpers::PropertyData<bool>(33, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'showLocations'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'hasOnlyPrinters'
        QtMocHelpers::PropertyData<bool>(35, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Role'
        QtMocHelpers::EnumData<enum Role>(36, 36, QMC::EnumFlags{}).add({
            {   37, Role::DestStatus },
            {   38, Role::DestState },
            {   39, Role::DestName },
            {   40, Role::DestIsDefault },
            {   41, Role::DestIsShared },
            {   42, Role::DestIsAcceptingJobs },
            {   43, Role::DestIsPaused },
            {   44, Role::DestIsClass },
            {   45, Role::DestLocation },
            {   46, Role::DestDescription },
            {   47, Role::DestKind },
            {   48, Role::DestType },
            {   49, Role::DestCommands },
            {   50, Role::DestMarkerChangeTime },
            {   51, Role::DestMarkers },
            {   52, Role::DestIconName },
            {   53, Role::DestRemote },
            {   54, Role::DestUri },
            {   55, Role::DestUriSupported },
            {   56, Role::DestMemberNames },
        }),
        // enum 'JobAction'
        QtMocHelpers::EnumData<enum JobAction>(57, 57, QMC::EnumFlags{}).add({
            {   58, JobAction::Cancel },
            {   59, JobAction::Hold },
            {   60, JobAction::Release },
            {   61, JobAction::Move },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<PrinterModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject PrinterModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QStandardItemModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PrinterModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PrinterModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12PrinterModelE_t>.metaTypes,
    nullptr
} };

void PrinterModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PrinterModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->serverUnavailableChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->error((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 2: _t->showLocationsChanged(); break;
        case 3: _t->hasOnlyPrintersChanged(); break;
        case 4: _t->update(); break;
        case 5: _t->getDestsFinished((*reinterpret_cast<std::add_pointer_t<KCupsRequest*>>(_a[1]))); break;
        case 6: _t->insertUpdatePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 7: _t->insertUpdatePrinterFinished((*reinterpret_cast<std::add_pointer_t<KCupsRequest*>>(_a[1]))); break;
        case 8: _t->printerRemoved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 9: _t->printerStateChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 10: _t->printerStopped((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 11: _t->printerRestarted((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 12: _t->printerShutdown((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 13: _t->printerModified((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<uint>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[6]))); break;
        case 14: _t->serverChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->pausePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->resumePrinter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PrinterModel::*)(bool )>(_a, &PrinterModel::serverUnavailableChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterModel::*)(int , const QString & , const QString & )>(_a, &PrinterModel::error, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterModel::*)()>(_a, &PrinterModel::showLocationsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PrinterModel::*)()>(_a, &PrinterModel::hasOnlyPrintersChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->serverUnavailable(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->showLocations(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->hasOnlyPrinters(); break;
        default: break;
        }
    }
}

const QMetaObject *PrinterModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PrinterModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12PrinterModelE_t>.strings))
        return static_cast<void*>(this);
    return QStandardItemModel::qt_metacast(_clname);
}

int PrinterModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QStandardItemModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 17;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void PrinterModel::serverUnavailableChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PrinterModel::error(int _t1, const QString & _t2, const QString & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void PrinterModel::showLocationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PrinterModel::hasOnlyPrintersChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
