/********************************************************************************
** Form generated from reading UI file 'PrinterOptions.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PRINTEROPTIONS_H
#define UI_PRINTEROPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_PrinterOptions
{
public:
    QGridLayout *gridLayout;
    QPushButton *autoConfigurePB;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QVBoxLayout *verticalLayout;

    void setupUi(QWidget *PrinterOptions)
    {
        if (PrinterOptions->objectName().isEmpty())
            PrinterOptions->setObjectName("PrinterOptions");
        PrinterOptions->resize(484, 427);
        gridLayout = new QGridLayout(PrinterOptions);
        gridLayout->setContentsMargins(0, 0, 0, 0);
        gridLayout->setObjectName("gridLayout");
        autoConfigurePB = new QPushButton(PrinterOptions);
        autoConfigurePB->setObjectName("autoConfigurePB");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(autoConfigurePB->sizePolicy().hasHeightForWidth());
        autoConfigurePB->setSizePolicy(sizePolicy);

        gridLayout->addWidget(autoConfigurePB, 0, 0, 1, 1);

        scrollArea = new QScrollArea(PrinterOptions);
        scrollArea->setObjectName("scrollArea");
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setWidgetResizable(true);
        scrollArea->setAlignment(Qt::AlignHCenter|Qt::AlignTop);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName("scrollAreaWidgetContents");
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 484, 401));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName("verticalLayout_2");
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");

        verticalLayout_2->addLayout(verticalLayout);

        scrollArea->setWidget(scrollAreaWidgetContents);

        gridLayout->addWidget(scrollArea, 1, 0, 1, 1);


        retranslateUi(PrinterOptions);

        QMetaObject::connectSlotsByName(PrinterOptions);
    } // setupUi

    void retranslateUi(QWidget *PrinterOptions)
    {
        autoConfigurePB->setText(tr2i18n("Query Printer for Default Options", nullptr));
        (void)PrinterOptions;
    } // retranslateUi

};

namespace Ui {
    class PrinterOptions: public Ui_PrinterOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // PRINTEROPTIONS_H

